@@ -1,5 +1,12 @@
 from firebolt.client.auth import Auth  # backward compatibility
-from firebolt.client.client import AsyncClient, Client
+from firebolt.client.client import (
+    AsyncClient,
+    AsyncClientV1,
+    AsyncClientV2,
+    Client,
+    ClientV1,
+    ClientV2,
+)
 from firebolt.client.constants import DEFAULT_API_URL
 from firebolt.client.resource_manager_hooks import (
     log_request,
