@@ -1,11 +1,13 @@
+from abc import abstractmethod
 from time import time
-from typing import Generator, Optional
+from typing import AsyncGenerator, Generator, Optional
 
+from anyio import Lock
 from httpx import Auth as HttpxAuth
 from httpx import Request, Response, codes
 
 from firebolt.utils.token_storage import TokenSecureStorage
-from firebolt.utils.util import cached_property
+from firebolt.utils.util import Timer, cached_property, get_internal_error_code
 
 
 class AuthRequest(Request):
@@ -35,6 +37,7 @@ class Auth(HttpxAuth):
         self._use_token_cache = use_token_cache
         self._token: Optional[str] = self._get_cached_token()
         self._expires: Optional[int] = None
+        self._lock = Lock()
 
     def copy(self) -> "Auth":
         """Make another auth object with same credentials.
@@ -42,7 +45,7 @@ class Auth(HttpxAuth):
         Returns:
             Auth: Auth object
         """
-        return Auth(self._use_token_cache)
+        return self.__class__(self._use_token_cache)
 
     @property
     def token(self) -> Optional[str]:
@@ -53,6 +56,14 @@ class Auth(HttpxAuth):
         """
         return self._token
 
+    def get_firebolt_version(self) -> int:
+        """Get Firebolt version from auth.
+
+        Returns:
+            int: Firebolt version
+        """
+        return -1
+
     @property
     def expired(self) -> bool:
         """Check if current token is expired.
@@ -93,6 +104,7 @@ class Auth(HttpxAuth):
         if self._token and self._expires:
             self._token_storage.cache_token(self._token, self._expires)
 
+    @abstractmethod
     def get_new_token_generator(self) -> Generator[Request, Response, None]:
         """Generate requests needed to create a new token session."""
 
@@ -107,15 +119,46 @@ class Auth(HttpxAuth):
         Yields:
             Request: Request required for auth flow
         """
-        if not self.token or self.expired:
-            yield from self.get_new_token_generator()
-            self._cache_token()
+        with Timer("[PERFORMANCE] Authentication "):
+            if not self.token or self.expired:
+                yield from self.get_new_token_generator()
+                self._cache_token()
 
-        request.headers["Authorization"] = f"Bearer {self.token}"
-
-        response = yield request
-
-        if response.status_code == codes.UNAUTHORIZED:
-            yield from self.get_new_token_generator()
             request.headers["Authorization"] = f"Bearer {self.token}"
-            yield request
+
+            response = yield request
+
+            if (
+                response.status_code == codes.UNAUTHORIZED
+                or get_internal_error_code(response) == codes.UNAUTHORIZED
+            ):
+                yield from self.get_new_token_generator()
+                request.headers["Authorization"] = f"Bearer {self.token}"
+                yield request
+
+    async def async_auth_flow(
+        self, request: Request
+    ) -> AsyncGenerator[Request, Response]:
+        """
+        Execute the authentication flow asynchronously.
+
+        Overridden in order to lock and ensure no more than
+        one authentication request is sent at a time. This
+        avoids excessive load on the auth server.
+        """
+        if self.requires_request_body:
+            await request.aread()
+
+        async with self._lock:
+            flow = self.auth_flow(request)
+            request = next(flow)
+
+            while True:
+                response = yield request
+                if self.requires_response_body:
+                    await response.aread()
+
+                try:
+                    request = flow.send(response)
+                except StopIteration:
+                    break
